@@ -10,10 +10,19 @@ from functools import reduce
 import jsbeautifier
 from strbuilder import BaseBuilder, SurroundBuilder
 
-from .visitor import Transformer, Visitor, VisitorContext
+from .visitor import Visitor, VisitorContext
 
 transformers: t.List['Transformer'] = []
 
+
+class JSScope(enum.Enum):
+    MODULE = 0
+    CLASS_FIELD = 1
+    FUNCTION_FIELD = 2
+    ARGUMENT = 3
+    F_STRING = 4
+    FOR_LOOP = 5
+
 BOOL_OP_TABLE = {
     ast.Or: '||',
     ast.And: '&&'
@@ -51,47 +60,48 @@ CONSTANT_TABLE = {
 }
 
 
-class JSScope(enum.Enum):
-    MODULE = 0
-    CLASS_FIELD = 1
-    FUNCTION_FIELD = 2
-    ARGUMENT = 3
-    F_STRING = 4
-    FOR_LOOP = 5
-
-
 @dataclass
 class JSVisitorContext(VisitorContext):
-    variables: t.List[str] = field(default_factory=list)
-    scope: JSScope = field(default=JSScope.MODULE)
     bool_op_table: t.Dict[type, str] = field(default_factory=BOOL_OP_TABLE.copy)
     unary_op_table: t.Dict[type, str] = field(default_factory=UNARY_OP_TABLE.copy)
     compare_op_table: t.Dict[t.Any, str] = field(default_factory=COMPARE_OP_TABLE.copy)
     operator_op_table: t.Dict[type, str] = field(default_factory=OPERATOR_OP_TABLE.copy)
     constant_table: t.Dict[t.Any, str] = field(default_factory=CONSTANT_TABLE.copy)
+    variables: t.List[str] = field(default_factory=list)
+    scope: JSScope = field(default=JSScope.MODULE)
     break_suffix: str = field(default='')
     parent: ast.AST = field(default=None)
     node: ast.AST = field(default=None)
+    has_yield: bool = field(default=False)
 
-    def copy(self, variables: t.Dict[str, t.Any] = None, node: t.Optional[ast.AST] = None, scope: t.Optional[JSScope] = None, break_suffix: t.Optional[str] = None) -> 'JSVisitorContext':
+    def copy(
+        self,
+        variables: t.Dict[str, t.Any] = None,
+        node: t.Optional[ast.AST] = None,
+        scope: t.Optional[JSScope] = None,
+        break_suffix: t.Optional[str] = None,
+        has_yield: t.Optional[bool] = None
+    ) -> 'JSVisitorContext':
         return JSVisitorContext(
-            variables=variables or self.variables,
-            scope=scope or self.scope,
             bool_op_table=self.bool_op_table,
             unary_op_table=self.unary_op_table,
             compare_op_table=self.compare_op_table,
             operator_op_table=self.operator_op_table,
+            variables=variables or self.variables,
+            scope=scope or self.scope,
             constant_table=self.constant_table,
             break_suffix=self.break_suffix if break_suffix is None else break_suffix,
             parent=self.node,
-            node=node or self.node
+            node=node or self.node,
+            has_yield=self.has_yield if has_yield is None else has_yield
         )
 
 
 class CodeGen(Visitor[JSVisitorContext]):
+
     def __init__(self, python_compatible: bool = False) -> None:
         self.python_compatible = python_compatible
-        super().__init__(JSVisitorContext, transformers=transformers)
+        super().__init__(JSVisitorContext)
 
     def sperator(self, ast: t.Union[t.List[ast.AST], ast.AST], context: t.Optional[JSVisitorContext] = None):
         if context.scope == JSScope.ARGUMENT:
@@ -116,6 +126,8 @@ class CodeGen(Visitor[JSVisitorContext]):
             .write('from')\
             .write(SurroundBuilder(node.module))\
             .build()
+    
+    # def visit_Comment(self, node: ast.PyCF_TYPE_COMMENTS)
 
     def visit_ClassDef(self, node: ast.ClassDef, ctx: JSVisitorContext):
         context = ctx.copy(scope=JSScope.CLASS_FIELD)
@@ -135,43 +147,62 @@ class CodeGen(Visitor[JSVisitorContext]):
         return builder.build()
 
     def visit_FunctionDef(self, node: ast.FunctionDef, ctx: JSVisitorContext):
-        context = ctx.copy()
-        context.scope = JSScope.FUNCTION_FIELD
+        context = ctx.copy(scope=JSScope.FUNCTION_FIELD)
+
+        body = self.visit(node.body, context)
 
         if ctx.scope == JSScope.CLASS_FIELD:
             return BaseBuilder()\
+                .write_if(context.has_yield, '*')\
                 .write(node.name)\
                 .write(f'{self.visit(node.args, ctx)}')\
                 .write(SurroundBuilder(surround='{}')
-                       .write(self.visit(node.body, context)))\
+                       .write(body))\
                 .build()
         builder = BaseBuilder()\
             .write_if(node.name not in ctx.variables, 'let')\
             .write(node.name)\
             .write('=')\
-            .write(f'{self.visit(node.args, ctx)} =>')\
+            .write_if(context.has_yield, 'function*')\
+            .write(f'{self.visit(node.args, ctx)}')\
+            .write_if(not context.has_yield, '=>')\
             .write(SurroundBuilder(surround='{}')
-                   .write(self.visit(node.body, context)))
+                   .write(body))
         ctx.variables.append(node.name)
         return builder.build()
 
     def visit_AsyncFunctionDef(self, node: ast.AsyncFunctionDef, ctx: JSVisitorContext):
         context = ctx.copy(scope=JSScope.FUNCTION_FIELD)
 
+        body = self.visit(node.body, context)
+
         if ctx.scope == JSScope.CLASS_FIELD:
             return BaseBuilder()\
+                .write_if(context.has_yield, '*')\
                 .write(node.name)\
                 .write(f'{self.visit(node.args, ctx)}')\
                 .write(SurroundBuilder(surround='{}')
-                       .write(self.visit(node.body, context)))\
+                       .write(body))\
                 .build()
-        return BaseBuilder('let')\
+        builder = BaseBuilder()\
+            .write_if(node.name not in ctx.variables, 'let')\
             .write(node.name)\
             .write('=')\
-            .write(f'async {self.visit(node.args, ctx)} =>')\
+            .write_if(context.has_yield, 'async function*', or_else='async')\
+            .write(f'{self.visit(node.args, ctx)}')\
+            .write_if(not context.has_yield, '=>')\
             .write(SurroundBuilder(surround='{}')
-                   .write(self.visit(node.body, context)))\
-            .build()
+                   .write(body))
+        ctx.variables.append(node.name)
+        return builder.build()
+    
+    def visit_Yield(self, node: ast.Yield, ctx: JSVisitorContext):
+        ctx.has_yield = True
+        return f'yield {self.visit(node.value, ctx)}'
+    
+    def visit_YieldFrom(self, node: ast.YieldFrom, ctx: JSVisitorContext):
+        ctx.has_yield = True
+        return f'yield* {self.visit(node.value, ctx)}'
 
     def visit_If(self, node: ast.If, ctx: JSVisitorContext):
         return BaseBuilder('if').write(SurroundBuilder(surround='()').write(self.visit(node.test, ctx))).write(SurroundBuilder(surround='{}').write(self.visit(node.body, ctx))).build()
@@ -184,7 +215,7 @@ class CodeGen(Visitor[JSVisitorContext]):
             ops.write(ctx.compare_op_table[op_type])
             if op_type == ast.NotIn:
                 invert = True
-            
+
         builder = BaseBuilder(self.visit(node.left, ctx))
         comparators = node.comparators
         while comparators:
@@ -291,10 +322,16 @@ class CodeGen(Visitor[JSVisitorContext]):
         return f'{self.visit(node.args, ctx)}=> {{return {self.visit(node.body, ctx)}}}'
 
     def visit_For(self, node: ast.For, ctx: JSVisitorContext):
+        target = BaseBuilder()
+        if isinstance(node.target, ast.Name):
+            if node.target.id not in ctx.variables:
+                target.write(f'let')
+        target.write(self.visit(node.target, ctx))
+
         return (
             BaseBuilder()
             .write_if(node.orelse, '_else: {')
-            .write(f'for ({self.visit(node.target, ctx)} of {self.visit(node.iter, ctx)})')
+            .write(f'for ({target.build()} of {self.visit(node.iter, ctx)})')
             .write(f"{{{self.visit(node.body, ctx.copy(break_suffix='_else' if node.orelse else ''))}}}")
             .write_if(node.orelse, f'{self.visit(node.orelse, ctx)} }}')
         ).build()
@@ -347,7 +384,13 @@ class CodeGen(Visitor[JSVisitorContext]):
     def visit_AnnAssign(self, node: ast.AnnAssign, ctx: JSVisitorContext):
         if isinstance(node.target, ast.Name):
             ctx.variables.append(node.target.id)
-        return BaseBuilder().write_if(not isinstance(node.target, ast.Attribute) and ctx.scope != JSScope.CLASS_FIELD, 'let').write(f'{self.visit(node.target, ctx)}').write_if(node.value is not None, f'={self.visit(node.value, ctx)}').build()
+        print(node.value is not None)
+        return (
+            BaseBuilder()
+            .write_if(not isinstance(node.target, ast.Attribute) and ctx.scope != JSScope.CLASS_FIELD, 'let')
+            .write(f'{self.visit(node.target, ctx)}')
+            .write_if(node.value is not None, lambda: f'={self.visit(node.value, ctx)}')
+        ).build()
 
     def visit_Pass(self, node: ast.Pass, ctx: JSVisitorContext):
         return '/* pass */'
@@ -405,7 +448,7 @@ class CodeGen(Visitor[JSVisitorContext]):
             .write(f'try {{{self.visit(node.body, ctx)}}}')
             .write_if(node.handlers, "catch(_err) {%s}" % '\n'.join(
                 BaseBuilder()
-                .write_if(handler.type, f'if (_err instanceof {self.visit(handler.type, ctx)})')
+                .write_if(handler.type, lambda: f'if (_err instanceof {self.visit(handler.type, ctx)})')
                 .write(SurroundBuilder(surround='{}', separator=';')
                        .write_if(handler.name, f'{handler.name} = _err')
                        .write(self.visit(handler.body, ctx))
